package test;

import static org.junit.jupiter.api.Assertions.assertEquals;

import java.math.BigInteger;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

import org.junit.jupiter.api.Test;

import com.cringe_studios.cringe_authenticator_library.OTP;
import com.cringe_studios.cringe_authenticator_library.OTPAlgorithm;
import com.cringe_studios.cringe_authenticator_library.OTPType;
import com.cringe_studios.cringe_authenticator_library.impl.Base32;
import com.cringe_studios.cringe_authenticator_library.impl.TOTP;

public class OTPTest {
	public static final boolean VERBOSE = true;

	@Test
	public void hotpTest() {
		String secret = "3132333435363738393031323334353637383930"; // Secret =
																	// 0x3132333435363738393031323334353637383930

		String base32secret = Base32.encode(hexStr2Bytes(secret));

		String[] expectedValues = { "755224", "287082", "359152", "969429", "338314", "254676", "287922", "162583",
				"399871", "520489" };
		
		if(VERBOSE) {
			System.out.println("calcPin   expected");
		}
		
		for (int i = 0; i < 10; i++) {
			OTP testOTP1 = OTPType.HOTP.instance(base32secret, OTPAlgorithm.SHA1, 6, i, 30, false);
			String calculatedPin = testOTP1.getPin();
			
			if(VERBOSE) {
				System.out.println(calculatedPin + " == " + expectedValues[i]);
			}
			
			assertEquals(calculatedPin, expectedValues[i]);
		}
	}

	private static byte[] hexStr2Bytes(String hex) {
		// Adding one byte to get the right conversion
		// Values starting with "0" can be converted
		byte[] bArray = new BigInteger("10" + hex, 16).toByteArray();

		// Copy all the REAL bytes, not the "first"
		byte[] ret = new byte[bArray.length - 1];
		for (int i = 0; i < ret.length; i++)
			ret[i] = bArray[i + 1];
		return ret;
	}
	
	@Test
	public void totpTest() {
		String expectedValues[] = {
				  "94287082",
				  "46119246",
				  "90693936",
				  "07081804",
				  "68084774",
				  "25091201",
				  "14050471",
				  "67062674",
				  "99943326",
				  "89005924",
				  "91819424",
				  "93441116",
				  "69279037",
				  "90698825",
				  "38618901",
				  "65353130",
				  "77737706",
				  "47863826"
		};
		
		// Seed for HMAC-SHA1 - 20 bytes
		String base16seed = "3132333435363738393031323334353637383930";
		// Seed for HMAC-SHA256 - 32 bytes
		String base16seed32 = "3132333435363738393031323334353637383930313233343536373839303132";
		// Seed for HMAC-SHA512 - 64 bytes
		String base16seed64 = "31323334353637383930313233343536373839303132333435363738393031323334353637383930313233343536373839303132333435363738393031323334";

		String seed = Base32.encode(hexStr2Bytes(base16seed));
		String seed32 = Base32.encode(hexStr2Bytes(base16seed32));
		String seed64 = Base32.encode(hexStr2Bytes(base16seed64));

		long testTime[] = { 59L, 1111111109L, 1111111111L, 1234567890L, 2000000000L, 20000000000L };

		TOTP testOTP1 = (TOTP) OTPType.TOTP.instance(seed, OTPAlgorithm.SHA1, 8, 0, 30, false);
		TOTP testOTP256 = (TOTP) OTPType.TOTP.instance(seed32, OTPAlgorithm.SHA256, 8, 0, 30, false);
		TOTP testOTP512 = (TOTP) OTPType.TOTP.instance(seed64, OTPAlgorithm.SHA512, 8, 0, 30, false);

		long X = 30;

		String steps = "0";
		DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		df.setTimeZone(TimeZone.getTimeZone("UTC"));

		try {
			if(VERBOSE) {
				System.out.println("+---------------+-----------------------+--------+--------+");
				System.out.println("|  Time(sec)    |   Time (UTC format)   |  TOTP  | Mode   |");
				System.out.println("+---------------+-----------------------+--------+--------+");
			}

			for (int i = 0; i < testTime.length; i++) {
				String fmtTime = String.format("%1$-11s", testTime[i]);
				String utcTime = df.format(new Date(testTime[i] * 1000));
				
				String sha1Pin = testOTP1.getPinAt(testTime[i]);
				String sha256Pin = testOTP256.getPinAt(testTime[i]);
				String sha512Pin = testOTP512.getPinAt(testTime[i]);
				
				
				
				
				if(VERBOSE) {
					System.out.println("|  " + fmtTime + "  |  " + utcTime + "  |" + sha1Pin + "| SHA1   |");
					System.out.println("|  " + fmtTime + "  |  " + utcTime + "  |" + sha256Pin + "| SHA256 |");
					System.out.println("|  " + fmtTime + "  |  " + utcTime + "  |" + sha512Pin + "| SHA512 |");
					System.out.println("+---------------+-----------------------+--------+--------+");
				}
				
				assertEquals(expectedValues[i * 3 + 0], sha1Pin);
				assertEquals(expectedValues[i * 3 + 1], sha256Pin);
				assertEquals(expectedValues[i * 3 + 2], sha512Pin);
			}
		} catch (final Exception e) {
			e.printStackTrace();
		}
	}

	// @org.junit.jupiter.api.Test
	// public void test() {
	// hotpTest();
	// totpTest();
	// assertEquals(1, 1);
	// }
}
